﻿// This source code is taylored to cater foobar2000.
// Modified from Expression Encoder 4 SDK example.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Expression.Encoder;
using System.IO;

namespace CommandEncoder
{
    class Program
    {
        static void Main(string[] args)
        {
            // Checks for number of arguments and advises of usages
            if (args.Length < 1)
            {
                Console.WriteLine();
                Console.WriteLine("! WMA encoder commandline interface for foobar2000");
                Console.WriteLine("! Require .NET Framework 4.0 and Expression Encoder 4");
                Console.WriteLine();
                //Console.WriteLine("Usage: CommandEncoder -j[d] <JobItem> [Destination] (not available)");
                Console.WriteLine("Usage: CommandEncoder -m[p][d] <MediaItem> [Preset] [Destination]");
                Console.WriteLine();
                Console.WriteLine(" [Preset]         In one of the following forms");
                Console.WriteLine("    <PresetFile>  Use the xml specified (uses encoder directory if no path)");
                Console.WriteLine("    <ShortName>   Choose one from WMALossless51Audio, WMABestQualityVBR, ");
                Console.WriteLine("                  WMAHighQualityAudio, WMAGoodQualityAudio, WMALowQualityAudio, ");
                Console.WriteLine("                  WMAVoiceAudio.");
                Console.WriteLine("    <PresetName>  Complete preset name in Microsoft.Expression.Encoder.dll");
                Console.WriteLine(" [Destination]    Complete file path, or file name only.");
                Console.WriteLine();
                Console.WriteLine("Example: ");
                Console.WriteLine(" CommandEncoder -m input.wav");
                Console.WriteLine(" CommandEncoder -md input.wav output.wma");
                Console.WriteLine(" CommandEncoder -md input.wav D:\\Test\\output.wma");
                Console.WriteLine(" CommandEncoder -mpd input.wav wmalossless.xml output.wma");
                Console.WriteLine(" CommandEncoder -mpd input.wav D:\\Test\\wmahigh.xml output.wma");
                Console.WriteLine(" CommandEncoder -mpd input.wav WMABestQualityVBR output.wma");
                Console.WriteLine(" CommandEncoder -mpd input.wav \"WMA High Quality Audio\" output.wma");
                return;
            }

            // converts string to lower case
            args[0] = args[0].ToLower();
            switch (args[0])
            {
                case "-j":
                    LoadJob(args[1], null);
                    break;
                
                case "-jd":
                    if (args.Length == 3 && Directory.Exists(args[2]))
                        LoadJob(args[1], args[2]);
                    else
                        Console.WriteLine("Invalid Number of Arguments for Command -jd");
                    break;

                case "-m":
                    LoadMedia(args[1], null, null);
                    break;

                case "-mp":
                    if(args.Length == 3)
                        LoadMedia(args[1], args[2], null);
                    else
                        Console.WriteLine("Invalid Number of Arguments for Command -mp");
                    break;

                case "-md":
                    if (args.Length == 3) // Output path of fb2k is a file, so directory check is removed
                        LoadMedia(args[1], null, args[2]);
                    else
                        Console.WriteLine("Invalid Number of Arguments for Command -md");
                    break;

                case "-mpd":
                    if (args.Length == 4) // Output path of fb2k is a file, so directory check is removed
                        LoadMedia(args[1], args[2], args[3]);
                    else
                        Console.WriteLine("Invalid Number of Arguments for Command -mpd");
                    break;

                default:
                    Console.WriteLine("Invalid Entry");
                    break;
            }
        }

        /// <summary>
        /// Verifies the job and loads it
        /// </summary>
        /// <param name="JobName"></param>
        /// <param name="Destination"></param>
        private static void LoadJob(string JobName, string Destination)
        {
            Job job;
            try
            {
                job = Job.LoadJob(JobName);
            }
            catch (InvalidJobException)
            {
                Console.WriteLine("Error: Invalid Job or Path");
                return;
            }

            Encode(job, null, Destination);
        }

        /// <summary>
        /// Verfies the media file, creates the job and adds the media file to the job
        /// </summary>
        /// <param name="MediaFile"></param>
        /// <param name="PresetName"></param>
        /// <param name="Destination"></param>
        private static void LoadMedia(string MediaFile, string PresetName, string Destination)
        {
            MediaItem mItem;
            try
            {
                mItem = new MediaItem(MediaFile);
            }
            catch (InvalidMediaFileException)
            {
                Console.WriteLine("Error: Invalid Media File or Path");
                return;
            }

            // Set file name as fb2k designates
            mItem.OutputFileName = Path.GetFileName(Destination);

            Job job = new Job();
            job.MediaItems.Add(mItem);

            Encode(job, PresetName, Destination);
        }

        /// <summary>
        /// Encodes the job using any passed presets and custom destinations
        /// </summary>
        /// <param name="job"></param>
        /// <param name="PresetName"></param>
        /// <param name="Destination"></param>
        private static void Encode(Job job, string PresetName, string Destination)
        {
            // Sets the destination to fb2k working folder
            job.OutputDirectory = Directory.GetCurrentDirectory();

            // Do not create subfolder with FileID
            job.CreateSubfolder = false;

            // Set up the progress callback function
            job.EncodeProgress += new EventHandler<EncodeProgressEventArgs>(OnProgress);

            //Sets preset if it was set and encodes job
            try
            {
                if (PresetName != null)
                {
                    // This part is modified from EncoderPowerShellModule
                    if (File.Exists(AppDomain.CurrentDomain.BaseDirectory.TrimEnd('\\') + "\\" + PresetName))
                    {
                        job.ApplyPreset(AppDomain.CurrentDomain.BaseDirectory.TrimEnd('\\') + "\\" + PresetName);
                    }
                    else if (File.Exists(PresetName))
                    {
                        job.ApplyPreset(PresetName);
                    }
                    else
                    {
                        switch (PresetName.ToUpperInvariant())
                        {
                            case "WMALOSSLESS51AUDIO":
                                job.ApplyPreset(Preset.FindPreset("WMA Lossless 5.1 Audio"));
                                break;
                            case "WMABESTQUALITYVBR":
                                job.ApplyPreset(Preset.FindPreset("WMA Best Quality VBR"));
                                break;
                            case "WMAHIGHQUALITYAUDIO":
                                job.ApplyPreset(Preset.FindPreset("WMA High Quality Audio"));
                                break;
                            case "WMAGOODQUALITYAUDIO":
                                job.ApplyPreset(Preset.FindPreset("WMA Good Quality Audio"));
                                break;
                            case "WMALOWQUALITYAUDIO":
                                job.ApplyPreset(Preset.FindPreset("WMA Low Quality Audio"));
                                break;
                            case "WMAVOICEAUDIO":
                                job.ApplyPreset(Preset.FindPreset("WMA Voice Audio"));
                                break;
                            default:
                                List<Preset> presetList = new List<Preset>(Microsoft.Expression.Encoder.Preset.SystemPresets);
                                for (int i = 0; i < presetList.Count; ++i)
                                {
                                    if (presetList[i].Name.ToUpperInvariant() == PresetName.ToUpperInvariant())
                                    {
                                        job.ApplyPreset(presetList[i]);
                                        break;
                                    }
                                }
                                break;
                        }
                    }
                }


                job.Encode();
            }
            catch (EncodeErrorException exp)
            {
                Console.WriteLine("Encoding Error: " + exp.ToString());
                return;
            }
            catch (PresetNotFoundException exp)
            {
                Console.WriteLine("Invalid Preset: " + exp.ToString());
                return;
            }
        }

        private static void OnProgress(object sender, EncodeProgressEventArgs e)
        {
            Console.Write("\b\b\b\b\b\b\b\b{0:F1}%", e.Progress);
        }
    }
}
